#ifndef MD_INTEGRATORNVU_H
#define MD_INTEGRATORNVU_H

/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "md_Integrator.h"
#include "md_sample.h"

class IntegratorNVU : public Integrator{
  
 private:
  void AllocateNVUState(int nb, float targetPotentialEnergy);
  void FreeNVUState();

  Sample* S; // The current sample.

  float* h_constraintForce;
  float* d_constraintForce; // Divide by displacementLength = Lagrange multiplier. 
  float* d_totalConstraintForceDenominator;

  float* h_previousPotentialEnergy; // Numerical correction.  
  float* d_previousPotentialEnergy; 

  float* h_constraintForceCorrection; // Numerical correction.
  float* d_constraintForceCorrection;

  float* h_displacementLengthCorrection; // Numerical correction. 
  float* d_displacementLengthCorrection; 

  float* d_constraintForceNumerator; // Arrays for summation.
  float* d_constraintForceDenominator; 

  float4* h_euclideanConfiguration;  
  float4* d_particleVelocity;

  float timeStep; 
  float displacementLength;
  float targetPotentialEnergy; // Per Particle.

  int numberOfBlocks; // For parallel summation.
  
 public:
  IntegratorNVU(Sample* S);	
  ~IntegratorNVU();
  
  void Integrate();
  void AllocateNVUState();
  void CalculateConstraintForce();
  void CalculateNumericalCorrection();
  
  // Set methods.
  void SetTimeStep(float dt);
  void SetDisplacementLength(float step);
  void SetTargetPotentialEnergy(float U);
  void SetSystemVelocityToZero();
  void SetEuclideanConfiguration();
  void SetEuclideanConfiguration(float4* c);

  // Get methods.
  float GetTimeStep();
  float GetSimulationTimeStepSq();
  float GetDisplacementLength();
  float GetSimulationDisplacementLength();
  float GetConstraintForceCorrection();
  float GetDisplacementLengthCorrection();
  float GetTargetPotentialEnergy();
  float GetLagrangeMultiplier();
  float GetEuclideanLength();
  
  double GetKineticEnergy();
};

#endif // MD_INTEGRATORNVU_H
